﻿#region License
//
// © Copyright Ray Hayes
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.
//
#endregion

#region Using directives

using System.ComponentModel;
using System.Diagnostics.CodeAnalysis;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;

#endregion

namespace Sentinel.Support
{
    public class GridViewSort
    {
        public static readonly DependencyProperty AutoSortProperty;

        public static readonly DependencyProperty CommandProperty;

        public static readonly DependencyProperty PropertyNameProperty =
            DependencyProperty.RegisterAttached(
                "PropertyName",
                typeof(string),
                typeof(GridViewSort),
                new UIPropertyMetadata(null));

        public static readonly DependencyProperty ShowSortGlyphProperty =
            DependencyProperty.RegisterAttached(
                "ShowSortGlyph",
                typeof(bool),
                typeof(GridViewSort),
                new UIPropertyMetadata(true));

        private static readonly DependencyProperty SortedColumnHeaderProperty =
            DependencyProperty.RegisterAttached(
                "SortedColumnHeader",
                typeof(GridViewColumnHeader),
                typeof(GridViewSort),
                new UIPropertyMetadata(null));

        public static readonly DependencyProperty SortGlyphAscendingProperty =
            DependencyProperty.RegisterAttached(
                "SortGlyphAscending",
                typeof(ImageSource),
                typeof(GridViewSort),
                new UIPropertyMetadata(null));

        public static readonly DependencyProperty SortGlyphDescendingProperty =
            DependencyProperty.RegisterAttached(
                "SortGlyphDescending",
                typeof(ImageSource),
                typeof(GridViewSort),
                new UIPropertyMetadata(null));

        static GridViewSort()
        {
            PropertyChangedCallback commandCallback =
                delegate(
                    DependencyObject o,
                    DependencyPropertyChangedEventArgs e)
                    {
                        ItemsControl listView = o as ItemsControl;
                        if (listView != null)
                        {
                            if (!GetAutoSort(listView))
                            {
                                if (e.OldValue != null &&
                                    e.NewValue == null)
                                {
                                    listView.RemoveHandler(
                                        ButtonBase.ClickEvent,
                                        new RoutedEventHandler(ColumnHeaderClick));
                                }

                                if (e.OldValue == null &&
                                    e.NewValue != null)
                                {
                                    listView.AddHandler(
                                        ButtonBase.ClickEvent,
                                        new RoutedEventHandler(ColumnHeaderClick));
                                }
                            }
                        }
                    };

            CommandProperty = DependencyProperty.RegisterAttached(
                "Command",
                typeof(ICommand),
                typeof(GridViewSort),
                new UIPropertyMetadata(null, commandCallback));

            PropertyChangedCallback autoSortCallback =
                delegate(
                    DependencyObject o,
                    DependencyPropertyChangedEventArgs e)
                    {
                        ListView listView = o as ListView;
                        if (listView != null)
                        {
                            // Don't change click handler if a command is set
                            if (GetCommand(listView) == null)
                            {
                                bool oldValue = (bool) e.OldValue;
                                bool newValue = (bool) e.NewValue;
                                if (oldValue && !newValue)
                                {
                                    listView.RemoveHandler(
                                        ButtonBase.ClickEvent,
                                        new RoutedEventHandler(ColumnHeaderClick));
                                }

                                if (!oldValue && newValue)
                                {
                                    listView.AddHandler(
                                        ButtonBase.ClickEvent,
                                        new RoutedEventHandler(ColumnHeaderClick));
                                }
                            }
                        }
                    };

            AutoSortProperty = DependencyProperty.RegisterAttached(
                "AutoSort",
                typeof(bool),
                typeof(GridViewSort),
                new UIPropertyMetadata(false, autoSortCallback));
        }

        public static void ApplySort(
            ICollectionView view,
            string propertyName,
            ListView listView,
            GridViewColumnHeader sortedColumnHeader)
        {
            ListSortDirection direction = ListSortDirection.Ascending;
            if (view.SortDescriptions.Count > 0)
            {
                SortDescription currentSort = view.SortDescriptions[0];
                if (currentSort.PropertyName == propertyName)
                {
                    direction = currentSort.Direction == ListSortDirection.Ascending
                                    ? ListSortDirection.Descending
                                    : ListSortDirection.Ascending;
                }

                view.SortDescriptions.Clear();

                GridViewColumnHeader currentSortedColumnHeader = GetSortedColumnHeader(listView);
                if (currentSortedColumnHeader != null)
                {
                    RemoveSortGlyph(currentSortedColumnHeader);
                }
            }

            if (!string.IsNullOrEmpty(propertyName))
            {
                view.SortDescriptions.Add(new SortDescription(propertyName, direction));
                if (GetShowSortGlyph(listView))
                {
                    ImageSource glyph = direction == ListSortDirection.Ascending
                                            ? GetSortGlyphAscending(listView)
                                            : GetSortGlyphDescending(listView);
                    AddSortGlyph(sortedColumnHeader, direction, glyph);
                }

                SetSortedColumnHeader(listView, sortedColumnHeader);
            }
        }

        [SuppressMessage(
            "Microsoft.Design",
            "CA1004:GenericMethodsShouldProvideTypeParameter",
            Justification = "The generic style registration is desired, despite this rule.")]
        public static T GetAncestor<T>(DependencyObject reference) where T : DependencyObject
        {
            DependencyObject parent = VisualTreeHelper.GetParent(reference);
            while (!(parent is T))
            {
                parent = VisualTreeHelper.GetParent(parent);
            }

            return parent != null ? (T) parent : null;
        }

        // Using a DependencyProperty as the backing store for Command.  This enables animation, styling, binding, etc...
        public static bool GetAutoSort(DependencyObject obj)
        {
            return (bool) obj.GetValue(AutoSortProperty);
        }

        public static ICommand GetCommand(DependencyObject obj)
        {
            return (ICommand) obj.GetValue(CommandProperty);
        }

        // Using a DependencyProperty as the backing store for AutoSort.  This enables animation, styling, binding, etc...
        public static string GetPropertyName(DependencyObject obj)
        {
            return (string) obj.GetValue(PropertyNameProperty);
        }

        // Using a DependencyProperty as the backing store for PropertyName.  This enables animation, styling, binding, etc...
        public static bool GetShowSortGlyph(DependencyObject obj)
        {
            return (bool) obj.GetValue(ShowSortGlyphProperty);
        }

        // Using a DependencyProperty as the backing store for ShowSortGlyph.  This enables animation, styling, binding, etc...
        public static ImageSource GetSortGlyphAscending(DependencyObject obj)
        {
            return (ImageSource) obj.GetValue(SortGlyphAscendingProperty);
        }

        // Using a DependencyProperty as the backing store for SortGlyphAscending.  This enables animation, styling, binding, etc...
        public static ImageSource GetSortGlyphDescending(DependencyObject obj)
        {
            return (ImageSource) obj.GetValue(SortGlyphDescendingProperty);
        }

        public static void SetAutoSort(DependencyObject obj, bool value)
        {
            obj.SetValue(AutoSortProperty, value);
        }

        public static void SetCommand(DependencyObject obj, ICommand value)
        {
            obj.SetValue(CommandProperty, value);
        }

        public static void SetPropertyName(DependencyObject obj, string value)
        {
            obj.SetValue(PropertyNameProperty, value);
        }

        public static void SetShowSortGlyph(DependencyObject obj, bool value)
        {
            obj.SetValue(ShowSortGlyphProperty, value);
        }

        public static void SetSortGlyphAscending(DependencyObject obj, ImageSource value)
        {
            obj.SetValue(SortGlyphAscendingProperty, value);
        }

        public static void SetSortGlyphDescending(DependencyObject obj, ImageSource value)
        {
            obj.SetValue(SortGlyphDescendingProperty, value);
        }

        private static void AddSortGlyph(
            GridViewColumnHeader columnHeader,
            ListSortDirection direction,
            ImageSource sortGlyph)
        {
            AdornerLayer adornerLayer = AdornerLayer.GetAdornerLayer(columnHeader);
            adornerLayer.Add(
                new SortGlyphAdorner(
                    columnHeader,
                    direction,
                    sortGlyph));
        }

        private static void ColumnHeaderClick(object sender, RoutedEventArgs e)
        {
            GridViewColumnHeader headerClicked = e.OriginalSource as GridViewColumnHeader;
            if (headerClicked != null && headerClicked.Column != null)
            {
                string propertyName = GetPropertyName(headerClicked.Column);
                if (!string.IsNullOrEmpty(propertyName))
                {
                    ListView listView = GetAncestor<ListView>(headerClicked);
                    if (listView != null)
                    {
                        ICommand command = GetCommand(listView);
                        if (command != null)
                        {
                            if (command.CanExecute(propertyName))
                            {
                                command.Execute(propertyName);
                            }
                        }
                        else if (GetAutoSort(listView))
                        {
                            lock (listView.Items)
                            {
                                ApplySort(listView.Items, propertyName, listView, headerClicked);
                            }
                        }
                    }
                }
            }
        }

        private static GridViewColumnHeader GetSortedColumnHeader(DependencyObject obj)
        {
            return (GridViewColumnHeader) obj.GetValue(SortedColumnHeaderProperty);
        }

        private static void RemoveSortGlyph(GridViewColumnHeader columnHeader)
        {
            AdornerLayer adornerLayer = AdornerLayer.GetAdornerLayer(columnHeader);
            Adorner[] adorners = adornerLayer.GetAdorners(columnHeader);
            if (adorners != null)
            {
                foreach (Adorner adorner in adorners)
                {
                    if (adorner is SortGlyphAdorner)
                    {
                        adornerLayer.Remove(adorner);
                    }
                }
            }
        }

        private static void SetSortedColumnHeader(DependencyObject obj, GridViewColumnHeader value)
        {
            obj.SetValue(SortedColumnHeaderProperty, value);
        }

        #region Nested type: SortGlyphAdorner

        private class SortGlyphAdorner : Adorner
        {
            private readonly GridViewColumnHeader columnHeader;

            private readonly ListSortDirection direction;

            private readonly ImageSource sortGlyph;

            public SortGlyphAdorner(
                GridViewColumnHeader columnHeader,
                ListSortDirection direction,
                ImageSource sortGlyph)
                : base(columnHeader)
            {
                this.columnHeader = columnHeader;
                this.direction = direction;
                this.sortGlyph = sortGlyph;
            }

            protected override void OnRender(DrawingContext drawingContext)
            {
                base.OnRender(drawingContext);

                if (sortGlyph != null)
                {
                    double x = columnHeader.ActualWidth - 13;
                    double y = (columnHeader.ActualHeight/2) - 5;
                    Rect rect = new Rect(x, y, 10, 10);
                    drawingContext.DrawImage(sortGlyph, rect);
                }
                else
                {
                    drawingContext.DrawGeometry(Brushes.LightGray, new Pen(Brushes.Gray, 1.0), GetDefaultGlyph());
                }
            }

            private Geometry GetDefaultGlyph()
            {
                double x1 = columnHeader.ActualWidth - 13;
                double x2 = x1 + 10;
                double x3 = x1 + 5;
                double y1 = (columnHeader.ActualHeight/2) - 3;
                double y2 = y1 + 5;

                if (direction == ListSortDirection.Ascending)
                {
                    double tmp = y1;
                    y1 = y2;
                    y2 = tmp;
                }

                PathSegmentCollection pathSegmentCollection =
                    new PathSegmentCollection
                        {
                            new LineSegment(new Point(x2, y1), true),
                            new LineSegment(new Point(x3, y2), true)
                        };

                PathFigure pathFigure = new PathFigure(new Point(x1, y1), pathSegmentCollection, true);

                PathFigureCollection pathFigureCollection = new PathFigureCollection();
                pathFigureCollection.Add(pathFigure);

                PathGeometry pathGeometry = new PathGeometry(pathFigureCollection);
                return pathGeometry;
            }
        }

        #endregion
    }
}